@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.10   2013/05/12
+ * Version: 0.1.11   2015/01/12
  */
 
 #ifndef _SECURITY_CAITSITH_INTERNAL_H
@@ -883,41 +883,29 @@ void cs_populate_patharg(struct cs_request_info *r, const bool first);
 void cs_transition_failed(const char *domainname);
 void cs_warn_oom(const char *function);
 int cs_chroot_permission(struct path *path);
-int cs_chmod_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			mode_t mode);
+int cs_chmod_permission(struct path *path, mode_t mode);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			kuid_t user, kgid_t group);
+int cs_chown_permission(struct path *path, kuid_t user, kgid_t group);
 #else
-int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			uid_t user, gid_t group);
+int cs_chown_permission(struct path *path, uid_t user, gid_t group);
 #endif
 int cs_fcntl_permission(struct file *file, unsigned int cmd,
 			unsigned long arg);
 int cs_ioctl_permission(struct file *filp, unsigned int cmd,
 			unsigned long arg);
-int cs_link_permission(struct dentry *old_dentry, struct dentry *new_dentry,
-		       struct vfsmount *mnt);
-int cs_mkdir_permission(struct dentry *dentry, struct vfsmount *mnt,
-			unsigned int mode);
-int cs_mknod_permission(struct dentry *dentry, struct vfsmount *mnt,
-			const unsigned int mode, unsigned int dev);
+int cs_link_permission(struct path *old, struct path *new);
+int cs_mkdir_permission(struct path *path, unsigned int mode);
+int cs_mknod_permission(struct path *path, const unsigned int mode,
+			unsigned int dev);
 int cs_mount_permission(const char *dev_name, struct path *path,
 			const char *type, unsigned long flags,
 			void *data_page);
 int cs_pivot_root_permission(struct path *old_path, struct path *new_path);
-#if 1
-int cs_rename_permission(struct dentry *old_dentry, struct dentry *new_dentry,
-			 struct vfsmount *mnt);
-#else
-int cs_rename_permission(struct dentry *old_dentry, struct dentry *new_dentry,
-			 struct vfsmount *mnt, unsigned int flags);
-#endif
-int cs_symlink_permission(struct dentry *dentry, struct vfsmount *mnt,
-			  const char *from);
-int cs_truncate_permission(struct dentry *dentry, struct vfsmount *mnt);
-int cs_umount_permission(struct vfsmount *mnt, int flags);
-int cs_unlink_permission(struct dentry *dentry, struct vfsmount *mnt);
+int cs_rename_permission(struct path *old, struct path *new);
+int cs_symlink_permission(struct path *path, const char *from);
+int cs_truncate_permission(struct path *path);
+int cs_umount_permission(struct path *path, int flags);
+int cs_unlink_permission(struct path *path);
 int cs_socket_create_permission(int family, int type, int protocol);
 int cs_socket_bind_permission(struct socket *sock, struct sockaddr *addr,
 			      int addr_len);
@@ -928,16 +916,10 @@ int cs_socket_post_accept_permission(struct socket *sock,
 				     struct socket *newsock);
 int cs_socket_sendmsg_permission(struct socket *sock,
 				 struct msghdr *msg, int size);
-int cs_rmdir_permission(struct dentry *dentry, struct vfsmount *mnt);
-int cs_getattr_permission(struct vfsmount *mnt, struct dentry *dentry);
+int cs_rmdir_permission(struct path *path);
+int cs_getattr_permission(struct path *path);
 bool cs_capable(const u8 operation);
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 32)
-int cs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
-		       const int flag);
-#else
-int cs_open_permission(struct file *filp);
-#endif
-
+int cs_open_permission(struct path *path, const int flag);
 
 /* Variable definition for internal use. */
 
