@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.10   2013/05/12
+ * Version: 0.1.11   2015/01/12
  */
 
 #ifndef _LINUX_CAITSITH_H
@@ -96,15 +96,9 @@ struct caitsith_operations {
 				   const char *from);
 	int (*truncate_permission) (struct dentry *dentry,
 				    struct vfsmount *mnt);
-#if 1
 	int (*rename_permission) (struct dentry *old_dentry,
 				  struct dentry *new_dentry,
 				  struct vfsmount *mnt);
-#else
-	int (*rename_permission) (struct dentry *old_dentry,
-				  struct dentry *new_dentry,
-				  struct vfsmount *mnt, unsigned int flags);
-#endif
 	int (*link_permission) (struct dentry *old_dentry,
 				struct dentry *new_dentry,
 				struct vfsmount *mnt);
@@ -297,7 +291,6 @@ static inline int ccs_truncate_permission(struct dentry *dentry,
 	return func ? func(dentry, mnt) : 0;
 }
 
-#if 1
 static inline int ccs_rename_permission(struct dentry *old_dentry,
 					struct dentry *new_dentry,
 					struct vfsmount *mnt)
@@ -306,17 +299,6 @@ static inline int ccs_rename_permission(struct dentry *old_dentry,
 		= caitsith_ops.rename_permission;
 	return func ? func(old_dentry, new_dentry, mnt) : 0;
 }
-#else
-static inline int ccs_rename_permission(struct dentry *old_dentry,
-					struct dentry *new_dentry,
-					struct vfsmount *mnt,
-					unsigned int flags)
-{
-	int (*func) (struct dentry *, struct dentry *, struct vfsmount *,
-		     unsigned int) = caitsith_ops.rename_permission;
-	return func ? func(old_dentry, new_dentry, mnt, flags) : 0;
-}
-#endif
 
 static inline int ccs_link_permission(struct dentry *old_dentry,
 				      struct dentry *new_dentry,
@@ -497,22 +479,12 @@ static inline int ccs_truncate_permission(struct dentry *dentry,
 	return 0;
 }
 
-#if 1
 static inline int ccs_rename_permission(struct dentry *old_dentry,
 					struct dentry *new_dentry,
 					struct vfsmount *mnt)
 {
 	return 0;
 }
-#else
-static inline int ccs_rename_permission(struct dentry *old_dentry,
-					struct dentry *new_dentry,
-					struct vfsmount *mnt,
-					unsigned int flags)
-{
-	return 0;
-}
-#endif
 
 static inline int ccs_link_permission(struct dentry *old_dentry,
 				      struct dentry *new_dentry,
