@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2010-2013  Tetsuo Handa <penguin-kernel@I-love.SAKURA.ne.jp>
  *
- * Version: 0.1.10   2013/05/12
+ * Version: 0.1.11   2015/01/12
  */
 
 #include "caitsith.h"
@@ -88,10 +88,6 @@ static struct security_operations original_security_ops /* = *security_ops; */;
  */
 #if defined(D_PATH_DISCONNECT)
 #define CS_INODE_HOOK_HAS_MNT
-#elif defined(CONFIG_SUSE_KERNEL) && LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 25)
-#define CS_INODE_HOOK_HAS_MNT
-#elif defined(CONFIG_SECURITY_APPARMOR) && LINUX_VERSION_CODE == KERNEL_VERSION(2, 6, 24)
-#define CS_INODE_HOOK_HAS_MNT
 #endif
 
 #if !defined(CONFIG_CAITSITH_DEBUG)
@@ -593,25 +589,7 @@ static void cs_bprm_free_security(struct linux_binprm *bprm)
 
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
-
-/**
- * cs_bprm_compute_creds - A hook which is called when do_execve() succeeded.
- *
- * @bprm: Pointer to "struct linux_binprm".
- *
- * Returns nothing.
- */
-static void cs_bprm_compute_creds(struct linux_binprm *bprm)
-{
-	if (bprm->security == &cs_bprm_security)
-		bprm->security = NULL;
-	while (!original_security_ops.bprm_compute_creds);
-	original_security_ops.bprm_compute_creds(bprm);
-	cs_clear_execve(0, cs_current_security());
-}
-
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29)
 
 /**
  * cs_bprm_apply_creds - A hook which is called when do_execve() succeeded.
@@ -766,8 +744,6 @@ static int cs_bprm_check_security(struct linux_binprm *bprm)
 	return original_security_ops.bprm_check_security(bprm);
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-
 /**
  * cs_open - Check permission for open().
  *
@@ -778,18 +754,14 @@ static int cs_bprm_check_security(struct linux_binprm *bprm)
 static int cs_open(struct file *f)
 {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-	return cs_open_permission(f);
+	return cs_open_permission(&f->f_path, f->f_flags);
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 6
-	return cs_open_permission(f->f_path.dentry, f->f_path.mnt,
-				  f->f_flags);
+	return cs_open_permission(&f->f_path.dentry, f->f_flags);
 #else
-	return cs_open_permission(f->f_path.dentry, f->f_path.mnt,
-				  f->f_flags + 1);
+	return cs_open_permission(&f->f_path, f->f_flags + 1);
 #endif
 }
 
-#endif
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
 
 /**
@@ -828,7 +800,7 @@ static int cs_dentry_open(struct file *f, const struct cred *cred)
 	return original_security_ops.dentry_open(f, cred);
 }
 
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#else
 
 /**
  * cs_dentry_open - Check permission for open().
@@ -846,68 +818,6 @@ static int cs_dentry_open(struct file *f)
 	return original_security_ops.dentry_open(f);
 }
 
-#else
-
-/**
- * cs_open - Check permission for open().
- *
- * @inode: Pointer to "struct inode".
- * @mask:  Open mode.
- * @nd:    Pointer to "struct nameidata".
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_open(struct inode *inode, int mask, struct nameidata *nd)
-{
-	int flags;
-	if (!nd || !nd->dentry)
-		return 0;
-	/* open_exec() passes MAY_EXEC . */
-	if (mask == MAY_EXEC && inode && S_ISREG(inode->i_mode) &&
-	    (cs_current_flags() & CS_TASK_IS_IN_EXECVE))
-		mask = MAY_READ;
-	/*
-	 * This flags value is passed to ACC_MODE().
-	 * cs_open_permission() for older versions uses old ACC_MODE().
-	 */
-	switch (mask & (MAY_READ | MAY_WRITE)) {
-	case MAY_READ:
-		flags = 01;
-		break;
-	case MAY_WRITE:
-		flags = 02;
-		break;
-	case MAY_READ | MAY_WRITE:
-		flags = 03;
-		break;
-	default:
-		return 0;
-	}
-	return cs_open_permission(nd->dentry, nd->mnt, flags);
-}
-
-/**
- * cs_inode_permission - Check permission for open().
- *
- * @inode: Pointer to "struct inode".
- * @mask:  Open mode.
- * @nd:    Pointer to "struct nameidata".
- *
- * Returns 0 on success, negative value otherwise.
- *
- * Note that this hook is called from permission(), and may not be called for
- * open(). Maybe it is better to use security_file_permission().
- */
-static int cs_inode_permission(struct inode *inode, int mask,
-			       struct nameidata *nd)
-{
-	int rc = cs_open(inode, mask, nd);
-	if (rc)
-		return rc;
-	while (!original_security_ops.inode_permission);
-	return original_security_ops.inode_permission(inode, mask, nd);
-}
-
 #endif
 
 #if defined(CONFIG_SECURITY_PATH)
@@ -925,7 +835,7 @@ static int cs_inode_permission(struct inode *inode, int mask,
  */
 static int cs_path_chown(struct path *path, kuid_t user, kgid_t group)
 {
-	int rc = cs_chown_permission(path->dentry, path->mnt, user, group);
+	int rc = cs_chown_permission(path, user, group);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_chown);
@@ -942,7 +852,7 @@ static int cs_path_chown(struct path *path, kuid_t user, kgid_t group)
  */
 static int cs_path_chmod(struct path *path, umode_t mode)
 {
-	int rc = cs_chmod_permission(path->dentry, path->mnt, mode);
+	int rc = cs_chmod_permission(path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_chmod);
@@ -978,7 +888,7 @@ static int cs_path_chroot(struct path *path)
  */
 static int cs_path_chown(struct path *path, uid_t user, gid_t group)
 {
-	int rc = cs_chown_permission(path->dentry, path->mnt, user, group);
+	int rc = cs_chown_permission(path, user, group);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_chown);
@@ -997,7 +907,7 @@ static int cs_path_chown(struct path *path, uid_t user, gid_t group)
  */
 static int cs_path_chmod(struct path *path, umode_t mode)
 {
-	int rc = cs_chmod_permission(path->dentry, path->mnt, mode);
+	int rc = cs_chmod_permission(path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_chmod);
@@ -1018,7 +928,8 @@ static int cs_path_chmod(struct path *path, umode_t mode)
 static int cs_path_chmod(struct dentry *dentry, struct vfsmount *vfsmnt,
 			 mode_t mode)
 {
-	int rc = cs_chmod_permission(dentry, vfsmnt, mode);
+	struct path path = { vfsmnt, dentry };
+	int rc = cs_chmod_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_chmod);
@@ -1056,7 +967,7 @@ static int cs_path_chroot(struct path *path)
  */
 static int cs_path_truncate(struct path *path)
 {
-	int rc = cs_truncate_permission(path->dentry, path->mnt);
+	int rc = cs_truncate_permission(path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_truncate);
@@ -1077,7 +988,7 @@ static int cs_path_truncate(struct path *path)
 static int cs_path_truncate(struct path *path, loff_t length,
 			    unsigned int time_attrs)
 {
-	int rc = cs_truncate_permission(path->dentry, path->mnt);
+	int rc = cs_truncate_permission(path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_truncate);
@@ -1104,16 +1015,17 @@ static int cs_inode_setattr(struct dentry *dentry, struct vfsmount *mnt,
 {
 	int rc = 0;
 #if !defined(CONFIG_SECURITY_PATH) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
+	struct path path = { mnt, dentry };
 	if (attr->ia_valid & ATTR_UID)
-		rc = cs_chown_permission(dentry, mnt, attr->ia_uid, -1);
+		rc = cs_chown_permission(&path, attr->ia_uid, -1);
 	if (!rc && (attr->ia_valid & ATTR_GID))
-		rc = cs_chown_permission(dentry, mnt, -1, attr->ia_gid);
+		rc = cs_chown_permission(&path, -1, attr->ia_gid);
 	if (!rc && (attr->ia_valid & ATTR_MODE))
-		rc = cs_chmod_permission(dentry, mnt, attr->ia_mode);
+		rc = cs_chmod_permission(&path, attr->ia_mode);
 #endif
 #if !defined(CONFIG_SECURITY_PATH)
 	if (!rc && (attr->ia_valid & ATTR_SIZE))
-		rc = cs_truncate_permission(dentry, mnt);
+		rc = cs_truncate_permission(&path);
 #endif
 	if (rc)
 		return rc;
@@ -1135,25 +1047,24 @@ static int cs_inode_setattr(struct dentry *dentry, struct iattr *attr)
 {
 	int rc = 0;
 #if !defined(CONFIG_SECURITY_PATH) || LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
+	struct path path = { NULL, dentry };
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
 	if (attr->ia_valid & ATTR_UID)
-		rc = cs_chown_permission(dentry, NULL, attr->ia_uid,
-					 INVALID_GID);
+		rc = cs_chown_permission(&path, attr->ia_uid, INVALID_GID);
 	if (!rc && (attr->ia_valid & ATTR_GID))
-		rc = cs_chown_permission(dentry, NULL, INVALID_UID,
-					 attr->ia_gid);
+		rc = cs_chown_permission(&path, INVALID_UID, attr->ia_gid);
 #else
 	if (attr->ia_valid & ATTR_UID)
-		rc = cs_chown_permission(dentry, NULL, attr->ia_uid, -1);
+		rc = cs_chown_permission(&path, attr->ia_uid, -1);
 	if (!rc && (attr->ia_valid & ATTR_GID))
-		rc = cs_chown_permission(dentry, NULL, -1, attr->ia_gid);
+		rc = cs_chown_permission(&path, -1, attr->ia_gid);
 #endif
 	if (!rc && (attr->ia_valid & ATTR_MODE))
-		rc = cs_chmod_permission(dentry, NULL, attr->ia_mode);
+		rc = cs_chmod_permission(&path, attr->ia_mode);
 #endif
 #if !defined(CONFIG_SECURITY_PATH)
 	if (!rc && (attr->ia_valid & ATTR_SIZE))
-		rc = cs_truncate_permission(dentry, NULL);
+		rc = cs_truncate_permission(&path);
 #endif
 	if (rc)
 		return rc;
@@ -1173,7 +1084,8 @@ static int cs_inode_setattr(struct dentry *dentry, struct iattr *attr)
  */
 static int cs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
 {
-	int rc = cs_getattr_permission(mnt, dentry);
+	struct path path = { mnt, dentry };
+	int rc = cs_getattr_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_getattr);
@@ -1197,7 +1109,8 @@ static int cs_inode_getattr(struct vfsmount *mnt, struct dentry *dentry)
 static int cs_path_mknod(struct path *dir, struct dentry *dentry,
 			 umode_t mode, unsigned int dev)
 {
-	int rc = cs_mknod_permission(dentry, dir->mnt, mode, dev);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_mknod_permission(&path, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mknod);
@@ -1216,7 +1129,8 @@ static int cs_path_mknod(struct path *dir, struct dentry *dentry,
 static int cs_path_mkdir(struct path *dir, struct dentry *dentry,
 			 umode_t mode)
 {
-	int rc = cs_mkdir_permission(dentry, dir->mnt, mode);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_mkdir_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mkdir);
@@ -1238,7 +1152,8 @@ static int cs_path_mkdir(struct path *dir, struct dentry *dentry,
 static int cs_path_mknod(struct path *dir, struct dentry *dentry, int mode,
 			 unsigned int dev)
 {
-	int rc = cs_mknod_permission(dentry, dir->mnt, mode, dev);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_mknod_permission(&path, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mknod);
@@ -1256,7 +1171,8 @@ static int cs_path_mknod(struct path *dir, struct dentry *dentry, int mode,
  */
 static int cs_path_mkdir(struct path *dir, struct dentry *dentry, int mode)
 {
-	int rc = cs_mkdir_permission(dentry, dir->mnt, mode);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_mkdir_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_mkdir);
@@ -1275,7 +1191,8 @@ static int cs_path_mkdir(struct path *dir, struct dentry *dentry, int mode)
  */
 static int cs_path_rmdir(struct path *dir, struct dentry *dentry)
 {
-	int rc = cs_rmdir_permission(dentry, dir->mnt);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_rmdir_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_rmdir);
@@ -1292,7 +1209,8 @@ static int cs_path_rmdir(struct path *dir, struct dentry *dentry)
  */
 static int cs_path_unlink(struct path *dir, struct dentry *dentry)
 {
-	int rc = cs_unlink_permission(dentry, dir->mnt);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_unlink_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_unlink);
@@ -1311,14 +1229,14 @@ static int cs_path_unlink(struct path *dir, struct dentry *dentry)
 static int cs_path_symlink(struct path *dir, struct dentry *dentry,
 			   const char *old_name)
 {
-	int rc = cs_symlink_permission(dentry, dir->mnt, old_name);
+	struct path path = { dir->mnt, dentry };
+	int rc = cs_symlink_permission(&path, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_symlink);
 	return original_security_ops.path_symlink(dir, dentry, old_name);
 }
 
-#if 1
 /**
  * cs_path_rename - Check permission for rename().
  *
@@ -1332,38 +1250,15 @@ static int cs_path_symlink(struct path *dir, struct dentry *dentry,
 static int cs_path_rename(struct path *old_dir, struct dentry *old_dentry,
 			  struct path *new_dir, struct dentry *new_dentry)
 {
-	int rc = cs_rename_permission(old_dentry, new_dentry, old_dir->mnt);
+	struct path old = { old_dir->mnt, old_dentry };
+	struct path new = { new_dir->mnt, new_dentry };
+	int rc = cs_rename_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_rename);
 	return original_security_ops.path_rename(old_dir, old_dentry, new_dir,
 						 new_dentry);
 }
-#else
-/**
- * cs_path_rename - Check permission for rename().
- *
- * @old_dir:    Pointer to "struct path".
- * @old_dentry: Pointer to "struct dentry".
- * @new_dir:    Pointer to "struct path".
- * @new_dentry: Pointer to "struct dentry".
- * @flags:      Rename flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_path_rename(struct path *old_dir, struct dentry *old_dentry,
-			  struct path *new_dir, struct dentry *new_dentry,
-			  unsigned int flags)
-{
-	int rc = cs_rename_permission(old_dentry, new_dentry, old_dir->mnt,
-				      flags);
-	if (rc)
-		return rc;
-	while (!original_security_ops.path_rename);
-	return original_security_ops.path_rename(old_dir, old_dentry, new_dir,
-						 new_dentry, flags);
-}
-#endif
 
 /**
  * cs_path_link - Check permission for link().
@@ -1377,7 +1272,9 @@ static int cs_path_rename(struct path *old_dir, struct dentry *old_dentry,
 static int cs_path_link(struct dentry *old_dentry, struct path *new_dir,
 			struct dentry *new_dentry)
 {
-	int rc = cs_link_permission(old_dentry, new_dentry, new_dir->mnt);
+	struct path old = { new_dir->mnt, old_dentry };
+	struct path new = { new_dir->mnt, new_dentry };
+	int rc = cs_link_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.path_link);
@@ -1401,7 +1298,8 @@ static int cs_path_link(struct dentry *old_dentry, struct path *new_dir,
 static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
 			  struct vfsmount *mnt, int mode, dev_t dev)
 {
-	int rc = cs_mknod_permission(dentry, mnt, mode, dev);
+	struct path path = { mnt, dentry };
+	int rc = cs_mknod_permission(&path, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mknod);
@@ -1421,7 +1319,8 @@ static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
 static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 			  struct vfsmount *mnt, int mode)
 {
-	int rc = cs_mkdir_permission(dentry, mnt, mode);
+	struct path path = { mnt, dentry };
+	int rc = cs_mkdir_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mkdir);
@@ -1440,7 +1339,8 @@ static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry,
 			  struct vfsmount *mnt)
 {
-	int rc = cs_rmdir_permission(dentry, mnt);
+	struct path path = { mnt, dentry };
+	int rc = cs_rmdir_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rmdir);
@@ -1459,7 +1359,8 @@ static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry,
 static int cs_inode_unlink(struct inode *dir, struct dentry *dentry,
 			   struct vfsmount *mnt)
 {
-	int rc = cs_unlink_permission(dentry, mnt);
+	struct path path = { mnt, dentry };
+	int rc = cs_unlink_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_unlink);
@@ -1479,7 +1380,8 @@ static int cs_inode_unlink(struct inode *dir, struct dentry *dentry,
 static int cs_inode_symlink(struct inode *dir, struct dentry *dentry,
 			    struct vfsmount *mnt, const char *old_name)
 {
-	int rc = cs_symlink_permission(dentry, mnt, old_name);
+	struct path path = { mnt, dentry };
+	int rc = cs_symlink_permission(&path, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_symlink);
@@ -1503,7 +1405,9 @@ static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 			   struct dentry *new_dentry,
 			   struct vfsmount *new_mnt)
 {
-	int rc = cs_rename_permission(old_dentry, new_dentry, new_mnt);
+	struct path old = { old_mnt, old_dentry };
+	struct path new = { new_mnt, new_dentry };
+	int rc = cs_rename_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rename);
@@ -1527,7 +1431,9 @@ static int cs_inode_link(struct dentry *old_dentry, struct vfsmount *old_mnt,
 			 struct inode *dir, struct dentry *new_dentry,
 			 struct vfsmount *new_mnt)
 {
-	int rc = cs_link_permission(old_dentry, new_dentry, new_mnt);
+	struct path old = { old_mnt, old_dentry };
+	struct path new = { new_mnt, new_dentry };
+	int rc = cs_link_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_link);
@@ -1548,7 +1454,8 @@ static int cs_inode_link(struct dentry *old_dentry, struct vfsmount *old_mnt,
 static int cs_inode_create(struct inode *dir, struct dentry *dentry,
 			   struct vfsmount *mnt, int mode)
 {
-	int rc = cs_mknod_permission(dentry, mnt, mode, 0);
+	struct path path = { mnt, dentry };
+	int rc = cs_mknod_permission(&path, mode, 0);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_create);
@@ -1572,7 +1479,8 @@ static int cs_inode_create(struct inode *dir, struct dentry *dentry,
 static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
 			  umode_t mode, dev_t dev)
 {
-	int rc = cs_mknod_permission(dentry, NULL, mode, dev);
+	struct path path = { NULL, dentry };
+	int rc = cs_mknod_permission(&path, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mknod);
@@ -1591,7 +1499,8 @@ static int cs_inode_mknod(struct inode *dir, struct dentry *dentry,
 static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 			  umode_t mode)
 {
-	int rc = cs_mkdir_permission(dentry, NULL, mode);
+	struct path path = { NULL, dentry };
+	int rc = cs_mkdir_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mkdir);
@@ -1613,7 +1522,8 @@ static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry,
 static int cs_inode_mknod(struct inode *dir, struct dentry *dentry, int mode,
 			  dev_t dev)
 {
-	int rc = cs_mknod_permission(dentry, NULL, mode, dev);
+	struct path path = { NULL, dentry };
+	int rc = cs_mknod_permission(&path, mode, dev);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mknod);
@@ -1631,7 +1541,8 @@ static int cs_inode_mknod(struct inode *dir, struct dentry *dentry, int mode,
  */
 static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry, int mode)
 {
-	int rc = cs_mkdir_permission(dentry, NULL, mode);
+	struct path path = { NULL, dentry };
+	int rc = cs_mkdir_permission(&path, mode);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_mkdir);
@@ -1650,7 +1561,8 @@ static int cs_inode_mkdir(struct inode *dir, struct dentry *dentry, int mode)
  */
 static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry)
 {
-	int rc = cs_rmdir_permission(dentry, NULL);
+	struct path path = { NULL, dentry };
+	int rc = cs_rmdir_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rmdir);
@@ -1667,7 +1579,8 @@ static int cs_inode_rmdir(struct inode *dir, struct dentry *dentry)
  */
 static int cs_inode_unlink(struct inode *dir, struct dentry *dentry)
 {
-	int rc = cs_unlink_permission(dentry, NULL);
+	struct path path = { NULL, dentry };
+	int rc = cs_unlink_permission(&path);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_unlink);
@@ -1686,14 +1599,14 @@ static int cs_inode_unlink(struct inode *dir, struct dentry *dentry)
 static int cs_inode_symlink(struct inode *dir, struct dentry *dentry,
 			    const char *old_name)
 {
-	int rc = cs_symlink_permission(dentry, NULL, old_name);
+	struct path path = { NULL, dentry };
+	int rc = cs_symlink_permission(&path, old_name);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_symlink);
 	return original_security_ops.inode_symlink(dir, dentry, old_name);
 }
 
-#if 1
 /**
  * cs_inode_rename - Check permission for rename().
  *
@@ -1707,37 +1620,15 @@ static int cs_inode_symlink(struct inode *dir, struct dentry *dentry,
 static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 			   struct inode *new_dir, struct dentry *new_dentry)
 {
-	int rc = cs_rename_permission(old_dentry, new_dentry, NULL);
+	struct path old = { NULL, old_dentry };
+	struct path new = { NULL, new_dentry };
+	int rc = cs_rename_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_rename);
 	return original_security_ops.inode_rename(old_dir, old_dentry, new_dir,
 						  new_dentry);
 }
-#else
-/**
- * cs_inode_rename - Check permission for rename().
- *
- * @old_dir:    Pointer to "struct inode".
- * @old_dentry: Pointer to "struct dentry".
- * @new_dir:    Pointer to "struct inode".
- * @new_dentry: Pointer to "struct dentry".
- * @flags:      Rename flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
-			   struct inode *new_dir, struct dentry *new_dentry,
-			   unsigned int flags)
-{
-	int rc = cs_rename_permission(old_dentry, new_dentry, NULL, flags);
-	if (rc)
-		return rc;
-	while (!original_security_ops.inode_rename);
-	return original_security_ops.inode_rename(old_dir, old_dentry, new_dir,
-						  new_dentry, flags);
-}
-#endif
 
 /**
  * cs_inode_link - Check permission for link().
@@ -1751,7 +1642,9 @@ static int cs_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
 static int cs_inode_link(struct dentry *old_dentry, struct inode *dir,
 			 struct dentry *new_dentry)
 {
-	int rc = cs_link_permission(old_dentry, new_dentry, NULL);
+	struct path old = { NULL, old_dentry };
+	struct path new = { NULL, new_dentry };
+	int rc = cs_link_permission(&old, &new);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_link);
@@ -1772,7 +1665,8 @@ static int cs_inode_link(struct dentry *old_dentry, struct inode *dir,
 static int cs_inode_create(struct inode *dir, struct dentry *dentry,
 			   umode_t mode)
 {
-	int rc = cs_mknod_permission(dentry, NULL, mode, 0);
+	struct path path = { NULL, dentry };
+	int rc = cs_mknod_permission(&path, mode, 0);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_create);
@@ -1793,7 +1687,8 @@ static int cs_inode_create(struct inode *dir, struct dentry *dentry,
 static int cs_inode_create(struct inode *dir, struct dentry *dentry,
 			   int mode)
 {
-	int rc = cs_mknod_permission(dentry, NULL, mode, 0);
+	struct path path = { NULL, dentry };
+	int rc = cs_mknod_permission(&path, mode, 0);
 	if (rc)
 		return rc;
 	while (!original_security_ops.inode_create);
@@ -1826,8 +1721,6 @@ static LIST_HEAD(cs_accepted_socket_list);
 /* Lock for protecting cs_accepted_socket_list . */
 static DEFINE_SPINLOCK(cs_accepted_socket_list_lock);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 8)
-
 /**
  * cs_socket_rcu_free - RCU callback for releasing "struct cs_socket_tag".
  *
@@ -1841,23 +1734,6 @@ static void cs_socket_rcu_free(struct rcu_head *rcu)
 	kfree(ptr);
 }
 
-#else
-
-/**
- * cs_socket_rcu_free - RCU callback for releasing "struct cs_socket_tag".
- *
- * @arg: Pointer to "void".
- *
- * Returns nothing.
- */
-static void cs_socket_rcu_free(void *arg)
-{
-	struct cs_socket_tag *ptr = arg;
-	kfree(ptr);
-}
-
-#endif
-
 /**
  * cs_update_socket_tag - Update tag associated with accept()ed sockets.
  *
@@ -1885,11 +1761,7 @@ static void cs_update_socket_tag(struct inode *inode, int status)
 		if (status)
 			break;
 		list_del_rcu(&ptr->list);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 8)
 		call_rcu(&ptr->rcu, cs_socket_rcu_free);
-#else
-		call_rcu(&ptr->rcu, cs_socket_rcu_free, ptr);
-#endif
 		break;
 	}
 	rcu_read_unlock();
@@ -2203,90 +2075,7 @@ static void cs_inode_free_security(struct inode *inode)
 
 #endif
 
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 24)
-
-/**
- * cs_sb_pivotroot - Check permission for pivot_root().
- *
- * @old_nd: Pointer to "struct nameidata".
- * @new_nd: Pointer to "struct nameidata".
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_sb_pivotroot(struct nameidata *old_nd, struct nameidata *new_nd)
-{
-	int rc = cs_pivot_root_permission(old_nd, new_nd);
-	if (rc)
-		return rc;
-	while (!original_security_ops.sb_pivotroot);
-	return original_security_ops.sb_pivotroot(old_nd, new_nd);
-}
-
-/**
- * cs_sb_mount - Check permission for mount().
- *
- * @dev_name:  Name of device file.
- * @nd:        Pointer to "struct nameidata".
- * @type:      Name of filesystem type. Maybe NULL.
- * @flags:     Mount options.
- * @data_page: Optional data. Maybe NULL.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_sb_mount(char *dev_name, struct nameidata *nd, char *type,
-		       unsigned long flags, void *data_page)
-{
-	int rc = cs_mount_permission(dev_name, nd, type, flags, data_page);
-	if (rc)
-		return rc;
-	while (!original_security_ops.sb_mount);
-	return original_security_ops.sb_mount(dev_name, nd, type, flags,
-					      data_page);
-}
-
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 26)
-
-/**
- * cs_sb_pivotroot - Check permission for pivot_root().
- *
- * @old_nd: Pointer to "struct nameidata".
- * @new_nd: Pointer to "struct nameidata".
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_sb_pivotroot(struct nameidata *old_nd, struct nameidata *new_nd)
-{
-	int rc = cs_pivot_root_permission(&old_nd->path, &new_nd->path);
-	if (rc)
-		return rc;
-	while (!original_security_ops.sb_pivotroot);
-	return original_security_ops.sb_pivotroot(old_nd, new_nd);
-}
-
-/**
- * cs_sb_mount - Check permission for mount().
- *
- * @dev_name:  Name of device file.
- * @nd:        Pointer to "struct nameidata".
- * @type:      Name of filesystem type. Maybe NULL.
- * @flags:     Mount options.
- * @data_page: Optional data. Maybe NULL.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int cs_sb_mount(char *dev_name, struct nameidata *nd, char *type,
-		       unsigned long flags, void *data_page)
-{
-	int rc = cs_mount_permission(dev_name, &nd->path, type, flags,
-				     data_page);
-	if (rc)
-		return rc;
-	while (!original_security_ops.sb_mount);
-	return original_security_ops.sb_mount(dev_name, nd, type, flags,
-					      data_page);
-}
-
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
 
 /**
  * cs_sb_pivotroot - Check permission for pivot_root().
@@ -2380,7 +2169,8 @@ static int cs_sb_mount(const char *dev_name, struct path *path,
  */
 static int cs_sb_umount(struct vfsmount *mnt, int flags)
 {
-	int rc = cs_umount_permission(mnt, flags);
+	struct path path = { mnt, mnt->mnt_root };
+	int rc = cs_umount_permission(&path, flags);
 	if (rc)
 		return rc;
 	while (!original_security_ops.sb_umount);
@@ -2425,7 +2215,7 @@ static int cs_file_ioctl(struct file *filp, unsigned int cmd,
 	return original_security_ops.file_ioctl(filp, cmd, arg);
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 21) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33) && defined(CONFIG_SYSCTL_SYSCALL)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33) && defined(CONFIG_SYSCTL_SYSCALL)
 
 /**
  * cs_prepend - Copy of prepend() in fs/dcache.c.
@@ -2549,9 +2339,7 @@ static void __init cs_update_security_ops(struct security_operations *ops)
 #endif
 	/* Security context updater for successful execve(). */
 	swap_security_ops(bprm_check_security);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
-	swap_security_ops(bprm_compute_creds);
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 29)
 	swap_security_ops(bprm_apply_creds);
 #else
 	swap_security_ops(bprm_committing_creds);
@@ -2559,14 +2347,12 @@ static void __init cs_update_security_ops(struct security_operations *ops)
 	/* Various permission checker. */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
 	swap_security_ops(file_open);
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
-	swap_security_ops(dentry_open);
 #else
-	swap_security_ops(inode_permission);
+	swap_security_ops(dentry_open);
 #endif
 	swap_security_ops(file_fcntl);
 	swap_security_ops(file_ioctl);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 21) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33) && defined(CONFIG_SYSCTL_SYSCALL)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33) && defined(CONFIG_SYSCTL_SYSCALL)
 	swap_security_ops(sysctl);
 #endif
 	swap_security_ops(sb_pivotroot);
@@ -2626,14 +2412,12 @@ static int __init cs_init(void)
 	struct security_operations *ops = probe_security_ops();
 	if (!ops)
 		goto out;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
 	caitsith_exports.find_task_by_vpid = probe_find_task_by_vpid();
 	if (!caitsith_exports.find_task_by_vpid)
 		goto out;
 	caitsith_exports.find_task_by_pid_ns = probe_find_task_by_pid_ns();
 	if (!caitsith_exports.find_task_by_pid_ns)
 		goto out;
-#endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 36)
 	caitsith_exports.vfsmount_lock = probe_vfsmount_lock();
 	if (!caitsith_exports.vfsmount_lock)
