@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.10   2013/05/12
+ * Version: 0.1.11   2015/01/12
  */
 
 #include "caitsith.h"
@@ -127,20 +127,16 @@ static bool cs_path_matches_group(const struct cs_path_info *pathname,
 static int cs_execute_path(struct linux_binprm *bprm, struct path *path);
 static int cs_execute(struct cs_request_info *r);
 static int cs_kern_path(const char *pathname, int flags, struct path *path);
-static int cs_mkdev_perm(const u8 operation, struct dentry *dentry,
-			 struct vfsmount *mnt, const unsigned int mode,
-			 unsigned int dev);
+static int cs_mkdev_perm(const u8 operation, struct path *path,
+			 const unsigned int mode, unsigned int dev);
 static int cs_mount_acl(const char *dev_name, struct path *dir,
 			const char *type, unsigned long flags,
 			const char *data);
-static int cs_path2_perm(const enum cs_mac_index operation,
-			 struct dentry *dentry1, struct vfsmount *mnt1,
-			 struct dentry *dentry2, struct vfsmount *mnt2);
-static int cs_path_number_perm(const enum cs_mac_index type,
-			       struct dentry *dentry, struct vfsmount *vfsmnt,
+static int cs_path2_perm(const enum cs_mac_index operation, struct path *path1,
+			 struct path *path2);
+static int cs_path_number_perm(const enum cs_mac_index type, struct path *path,
 			       unsigned long number);
-static int cs_path_perm(const enum cs_mac_index operation,
-			struct dentry *dentry, struct vfsmount *mnt);
+static int cs_path_perm(const enum cs_mac_index operation, struct path *path);
 int cs_start_execve(struct linux_binprm *bprm, struct cs_request_info **rp);
 static void cs_check_auto_domain_transition(void);
 static void cs_clear_request_info(struct cs_request_info *r);
@@ -589,6 +585,8 @@ static int cs_try_alt_exec(struct cs_request_info *r)
 	/* Close the requested program's dentry. */
 	r->obj.path[0].dentry = NULL;
 	r->obj.path[0].mnt = NULL;
+	r->obj.stat_valid[CS_PATH1] = false;
+	r->obj.stat_valid[CS_PATH1_PARENT] = false;
 	r->obj.validate_done = false;
 	allow_write_access(bprm->file);
 	fput(bprm->file);
@@ -731,8 +729,7 @@ static int cs_try_alt_exec(struct cs_request_info *r)
 		retval = PTR_ERR(filp);
 		goto out;
 	}
-	r->obj.path[0].dentry = filp->f_dentry;
-	r->obj.path[0].mnt = filp->f_vfsmnt;
+	r->obj.path[0] = filp->f_path;
 	bprm->file = filp;
 	bprm->filename = r->handler;
 	bprm->interp = bprm->filename;
@@ -842,8 +839,7 @@ int cs_start_execve(struct linux_binprm *bprm, struct cs_request_info **rp)
 	task->cs_flags &= ~CS_TASK_IS_MANAGER;
 	*rp = r;
 	r->bprm = bprm;
-	r->obj.path[0].dentry = bprm->file->f_dentry;
-	r->obj.path[0].mnt = bprm->file->f_vfsmnt;
+	r->obj.path[0] = bprm->file->f_path;
 	retval = cs_execute(r);
 #ifdef CONFIG_CAITSITH_ENVIRON
 	if (!retval && bprm->envc)
@@ -937,8 +933,8 @@ static int cs_execute_path(struct linux_binprm *bprm, struct path *path)
 	 * /proc/\$/exe is meaningless.
 	 */
 	const unsigned int follow =
-		(bprm->file->f_dentry->d_sb->s_magic == PROC_SUPER_MAGIC) ?
-		LOOKUP_FOLLOW : 0;
+		(bprm->file->f_path.dentry->d_sb->s_magic == PROC_SUPER_MAGIC)
+		? LOOKUP_FOLLOW : 0;
 	if (cs_kern_path(bprm->filename, follow, path))
 		return -ENOENT;
 	return 0;
@@ -1099,57 +1095,15 @@ int cs_mount_permission(const char *dev_name, struct path *path,
 	return cs_mount_acl(dev_name, path, type, flags, data_page);
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-
 /**
  * cs_open_permission - Check permission for "read" and "write".
  *
- * @filp: Pointer to "struct file".
+ * @path: Pointer to "struct path".
+ * @flag: Flags for open().
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_open_permission(struct file *filp)
-{
-	struct cs_request_info r = { };
-	const u32 cs_flags = cs_current_flags();
-	const u8 acc_mode = (filp->f_flags & 3) == 3 ?
-		0 : ACC_MODE(filp->f_flags);
-	int error = 0;
-	if (current->in_execve && !(cs_flags & CS_TASK_IS_IN_EXECVE))
-		return 0;
-#ifndef CONFIG_CAITSITH_GETATTR
-	if (dentry->d_inode && S_ISDIR(dentry->d_inode->i_mode))
-		return 0;
-#endif
-	r.obj.path[0] = filp->f_path;
-	if (!(cs_flags & CS_TASK_IS_IN_EXECVE))
-		cs_check_auto_domain_transition();
-	if (acc_mode & MAY_READ) {
-		r.type = CS_MAC_READ;
-		error = cs_check_acl(&r, false);
-	}
-	if (!error && (acc_mode & MAY_WRITE)) {
-		r.type = (filp->f_flags & O_APPEND) ?
-			CS_MAC_APPEND : CS_MAC_WRITE;
-		error = cs_check_acl(&r, false);
-	}
-	cs_clear_request_info(&r);
-	return error;
-}
-
-#else
-
-/**
- * cs_open_permission - Check permission for "read" and "write".
- *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
- * @flag:   Flags for open().
- *
- * Returns 0 on success, negative value otherwise.
- */
-int cs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
-		       const int flag)
+int cs_open_permission(struct path *path, const int flag)
 {
 	struct cs_request_info r = { };
 	const u32 cs_flags = cs_current_flags();
@@ -1165,11 +1119,10 @@ int cs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
 		return 0;
 #endif
 #ifndef CONFIG_CAITSITH_GETATTR
-	if (dentry->d_inode && S_ISDIR(dentry->d_inode->i_mode))
+	if (path->dentry->d_inode && S_ISDIR(path->dentry->d_inode->i_mode))
 		return 0;
 #endif
-	r.obj.path[0].dentry = dentry;
-	r.obj.path[0].mnt = mnt;
+	r.obj.path[0] = *path;
 	if (!(cs_flags & CS_TASK_IS_IN_EXECVE))
 		cs_check_auto_domain_transition();
 	if (acc_mode & MAY_READ) {
@@ -1190,25 +1143,20 @@ int cs_open_permission(struct dentry *dentry, struct vfsmount *mnt,
 	return error;
 }
 
-#endif
-
 /**
  * cs_path_perm - Check permission for "unlink", "rmdir", "truncate", "append", "getattr" and "chroot".
  *
  * @operation: One of values in "enum cs_mac_index".
- * @dentry:    Pointer to "struct dentry".
- * @mnt:       Pointer to "struct vfsmount". Maybe NULL.
+ * @path:      Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path_perm(const enum cs_mac_index operation,
-			struct dentry *dentry, struct vfsmount *mnt)
+static int cs_path_perm(const enum cs_mac_index operation, struct path *path)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
 	r.type = operation;
-	r.obj.path[0].dentry = dentry;
-	r.obj.path[0].mnt = mnt;
+	r.obj.path[0] = *path;
 	return cs_check_acl(&r, true);
 }
 
@@ -1216,21 +1164,18 @@ static int cs_path_perm(const enum cs_mac_index operation,
  * cs_mkdev_perm - Check permission for "mkblock" and "mkchar".
  *
  * @operation: Type of operation. (CS_MAC_MKCHAR or CS_MAC_MKBLOCK)
- * @dentry:    Pointer to "struct dentry".
- * @mnt:       Pointer to "struct vfsmount". Maybe NULL.
+ * @path:      Pointer to "struct path".
  * @mode:      Create mode.
  * @dev:       Device number.
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_mkdev_perm(const u8 operation, struct dentry *dentry,
-			 struct vfsmount *mnt, const unsigned int mode,
-			 unsigned int dev)
+static int cs_mkdev_perm(const u8 operation, struct path *path,
+			 const unsigned int mode, unsigned int dev)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
-	r.obj.path[0].dentry = dentry;
-	r.obj.path[0].mnt = mnt;
+	r.obj.path[0] = *path;
 #ifdef CONFIG_SECURITY_PATH
 	dev = new_decode_dev(dev);
 #endif
@@ -1245,44 +1190,36 @@ static int cs_mkdev_perm(const u8 operation, struct dentry *dentry,
  * cs_path2_perm - Check permission for "rename", "link" and "pivot_root".
  *
  * @operation: One of values in "enum cs_mac_index".
- * @dentry1:   Pointer to "struct dentry".
- * @mnt1:      Pointer to "struct vfsmount". Maybe NULL.
- * @dentry2:   Pointer to "struct dentry".
- * @mnt2:      Pointer to "struct vfsmount". Maybe NULL.
+ * @path1:     Pointer to "struct path".
+ * @path2:     Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path2_perm(const enum cs_mac_index operation,
-			 struct dentry *dentry1, struct vfsmount *mnt1,
-			 struct dentry *dentry2, struct vfsmount *mnt2)
+static int cs_path2_perm(const enum cs_mac_index operation, struct path *path1,
+			 struct path *path2)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
 	r.type = operation;
-	r.obj.path[0].dentry = dentry1;
-	r.obj.path[0].mnt = mnt1;
-	r.obj.path[1].dentry = dentry2;
-	r.obj.path[1].mnt = mnt2;
+	r.obj.path[0] = *path1;
+	r.obj.path[1] = *path2;
 	return cs_check_acl(&r, true);
 }
 
 /**
  * cs_symlink_permission - Check permission for "symlink".
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
+ * @path:   Pointer to "struct path".
  * @target: Content of symlink.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_symlink_permission(struct dentry *dentry, struct vfsmount *mnt,
-			  const char *target)
+int cs_symlink_permission(struct path *path, const char *target)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
 	r.type = CS_MAC_SYMLINK;
-	r.obj.path[0].dentry = dentry;
-	r.obj.path[0].mnt = mnt;
+	r.obj.path[0] = *path;
 	r.obj.pathname[1].name = cs_encode(target);
 	if (!r.obj.pathname[1].name)
 		return -ENOMEM;
@@ -1295,21 +1232,18 @@ int cs_symlink_permission(struct dentry *dentry, struct vfsmount *mnt,
  * cs_path_number_perm - Check permission for "create", "mkdir", "mkfifo", "mksock", "ioctl", "chmod", "chown", "chgrp" and "unmount".
  *
  * @type:   One of values in "enum cs_mac_index".
- * @dentry: Pointer to "struct dentry".
- * @vfsmnt: Pointer to "struct vfsmount". Maybe NULL.
+ * @path:   Pointer to "struct path".
  * @number: Number.
  *
  * Returns 0 on success, negative value otherwise.
  */
-static int cs_path_number_perm(const enum cs_mac_index type,
-			       struct dentry *dentry, struct vfsmount *vfsmnt,
+static int cs_path_number_perm(const enum cs_mac_index type, struct path *path,
 			       unsigned long number)
 {
 	struct cs_request_info r = { };
 	cs_check_auto_domain_transition();
 	r.type = type;
-	r.obj.path[0].dentry = dentry;
-	r.obj.path[0].mnt = vfsmnt;
+	r.obj.path[0] = *path;
 	r.param.i[0] = number;
 	return cs_check_acl(&r, true);
 }
@@ -1326,26 +1260,20 @@ static int cs_path_number_perm(const enum cs_mac_index type,
 int cs_ioctl_permission(struct file *filp, unsigned int cmd,
 			unsigned long arg)
 {
-	return cs_path_number_perm(CS_MAC_IOCTL, filp->f_dentry,
-				   filp->f_vfsmnt, cmd);
+	return cs_path_number_perm(CS_MAC_IOCTL, &filp->f_path, cmd);
 }
 
 /**
  * cs_chmod_permission - Check permission for "chmod".
  *
- * @dentry: Pointer to "struct dentry".
- * @vfsmnt: Pointer to "struct vfsmount". Maybe NULL.
- * @mode:   Mode.
+ * @path: Pointer to "struct path".
+ * @mode: Mode.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chmod_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			mode_t mode)
+int cs_chmod_permission(struct path *path, mode_t mode)
 {
-	if (mode == (mode_t) -1)
-		return 0;
-	return cs_path_number_perm(CS_MAC_CHMOD, dentry, vfsmnt,
-				   mode & S_IALLUGO);
+	return cs_path_number_perm(CS_MAC_CHMOD, path, mode & S_IALLUGO);
 }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
@@ -1353,22 +1281,20 @@ int cs_chmod_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
 /**
  * cs_chown_permission - Check permission for "chown/chgrp".
  *
- * @dentry: Pointer to "struct dentry".
- * @vfsmnt: Pointer to "struct vfsmount". Maybe NULL.
- * @user:   User ID.
- * @group:  Group ID.
+ * @path:  Pointer to "struct path".
+ * @user:  User ID.
+ * @group: Group ID.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			kuid_t user, kgid_t group)
+int cs_chown_permission(struct path *path, kuid_t user, kgid_t group)
 {
 	int error = 0;
 	if (uid_valid(user))
-		error = cs_path_number_perm(CS_MAC_CHOWN, dentry, vfsmnt,
+		error = cs_path_number_perm(CS_MAC_CHOWN, path,
 					    from_kuid(&init_user_ns, user));
 	if (!error && gid_valid(group))
-		error = cs_path_number_perm(CS_MAC_CHGRP, dentry, vfsmnt,
+		error = cs_path_number_perm(CS_MAC_CHGRP, path,
 					    from_kgid(&init_user_ns, group));
 	return error;
 }
@@ -1378,25 +1304,21 @@ int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
 /**
  * cs_chown_permission - Check permission for "chown/chgrp".
  *
- * @dentry: Pointer to "struct dentry".
- * @vfsmnt: Pointer to "struct vfsmount". Maybe NULL.
- * @user:   User ID.
- * @group:  Group ID.
+ * @path:  Pointer to "struct path".
+ * @user:  User ID.
+ * @group: Group ID.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_chown_permission(struct dentry *dentry, struct vfsmount *vfsmnt,
-			uid_t user, gid_t group)
+int cs_chown_permission(struct path *path, uid_t user, gid_t group)
 {
 	int error = 0;
 	if (user == (uid_t) -1 && group == (gid_t) -1)
 		return 0;
 	if (user != (uid_t) -1)
-		error = cs_path_number_perm(CS_MAC_CHOWN, dentry,
-					    vfsmnt, user);
+		error = cs_path_number_perm(CS_MAC_CHOWN, path, user);
 	if (!error && group != (gid_t) -1)
-		error = cs_path_number_perm(CS_MAC_CHGRP, dentry,
-					    vfsmnt, group);
+		error = cs_path_number_perm(CS_MAC_CHGRP, path, group);
 	return error;
 }
 
@@ -1417,12 +1339,11 @@ int cs_fcntl_permission(struct file *file, unsigned int cmd,
 	if (!(cmd == F_SETFL && ((arg ^ file->f_flags) & O_APPEND)))
 		return 0;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-	return cs_open_permission(file);
+	return cs_open_permission(&file->f_path, O_WRONLY | (arg & O_APPEND));
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 6
-	return cs_open_permission(file->f_dentry, file->f_vfsmnt,
-				  O_WRONLY | (arg & O_APPEND));
+	return cs_open_permission(&file->f_path, O_WRONLY | (arg & O_APPEND));
 #else
-	return cs_open_permission(file->f_dentry, file->f_vfsmnt,
+	return cs_open_permission(&file->f_path,
 				  (O_WRONLY + 1) | (arg & O_APPEND));
 #endif
 }
@@ -1437,8 +1358,7 @@ int cs_fcntl_permission(struct file *file, unsigned int cmd,
  */
 int cs_pivot_root_permission(struct path *old_path, struct path *new_path)
 {
-	return cs_path2_perm(CS_MAC_PIVOT_ROOT, new_path->dentry,
-			     new_path->mnt, old_path->dentry, old_path->mnt);
+	return cs_path2_perm(CS_MAC_PIVOT_ROOT, new_path, old_path);
 }
 
 /**
@@ -1450,58 +1370,52 @@ int cs_pivot_root_permission(struct path *old_path, struct path *new_path)
  */
 int cs_chroot_permission(struct path *path)
 {
-	return cs_path_perm(CS_MAC_CHROOT, path->dentry, path->mnt);
+	return cs_path_perm(CS_MAC_CHROOT, path);
 }
 
 /**
  * cs_umount_permission - Check permission for unmount.
  *
- * @mnt:   Pointer to "struct vfsmount".
+ * @path:  Pointer to "struct path".
  * @flags: Unmount flags.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_umount_permission(struct vfsmount *mnt, int flags)
+int cs_umount_permission(struct path *path, int flags)
 {
-	return cs_path_number_perm(CS_MAC_UMOUNT, mnt->mnt_root, mnt, flags);
+	return cs_path_number_perm(CS_MAC_UMOUNT, path, flags);
 }
 
 /**
  * cs_mknod_permission - Check permission for vfs_mknod().
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
- * @mode:   Device type and permission.
- * @dev:    Device number for block or character device.
+ * @path: Pointer to "struct path".
+ * @mode: Device type and permission.
+ * @dev:  Device number for block or character device.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_mknod_permission(struct dentry *dentry, struct vfsmount *mnt,
-			const unsigned int mode, unsigned int dev)
+int cs_mknod_permission(struct path *path, const unsigned int mode,
+			unsigned int dev)
 {
 	int error = 0;
 	const unsigned int perm = mode & S_IALLUGO;
 	switch (mode & S_IFMT) {
 	case S_IFCHR:
-		error = cs_mkdev_perm(CS_MAC_MKCHAR, dentry, mnt, perm,
-				      dev);
+		error = cs_mkdev_perm(CS_MAC_MKCHAR, path, perm, dev);
 		break;
 	case S_IFBLK:
-		error = cs_mkdev_perm(CS_MAC_MKBLOCK, dentry, mnt, perm,
-				      dev);
+		error = cs_mkdev_perm(CS_MAC_MKBLOCK, path, perm, dev);
 		break;
 	case S_IFIFO:
-		error = cs_path_number_perm(CS_MAC_MKFIFO, dentry, mnt,
-					    perm);
+		error = cs_path_number_perm(CS_MAC_MKFIFO, path, perm);
 		break;
 	case S_IFSOCK:
-		error = cs_path_number_perm(CS_MAC_MKSOCK, dentry, mnt,
-					    perm);
+		error = cs_path_number_perm(CS_MAC_MKSOCK, path, perm);
 		break;
 	case 0:
 	case S_IFREG:
-		error = cs_path_number_perm(CS_MAC_CREATE, dentry, mnt,
-					    perm);
+		error = cs_path_number_perm(CS_MAC_CREATE, path, perm);
 		break;
 	}
 	return error;
@@ -1510,42 +1424,38 @@ int cs_mknod_permission(struct dentry *dentry, struct vfsmount *mnt,
 /**
  * cs_mkdir_permission - Check permission for vfs_mkdir().
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
- * @mode:   Create mode.
+ * @path: Pointer to "struct path".
+ * @mode: Create mode.
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_mkdir_permission(struct dentry *dentry, struct vfsmount *mnt,
-			unsigned int mode)
+int cs_mkdir_permission(struct path *path, unsigned int mode)
 {
-	return cs_path_number_perm(CS_MAC_MKDIR, dentry, mnt, mode);
+	return cs_path_number_perm(CS_MAC_MKDIR, path, mode);
 }
 
 /**
  * cs_rmdir_permission - Check permission for vfs_rmdir().
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
+ * @path: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_rmdir_permission(struct dentry *dentry, struct vfsmount *mnt)
+int cs_rmdir_permission(struct path *path)
 {
-	return cs_path_perm(CS_MAC_RMDIR, dentry, mnt);
+	return cs_path_perm(CS_MAC_RMDIR, path);
 }
 
 /**
  * cs_unlink_permission - Check permission for vfs_unlink().
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
+ * @path: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_unlink_permission(struct dentry *dentry, struct vfsmount *mnt)
+int cs_unlink_permission(struct path *path)
 {
-	return cs_path_perm(CS_MAC_UNLINK, dentry, mnt);
+	return cs_path_perm(CS_MAC_UNLINK, path);
 }
 
 #ifdef CONFIG_CAITSITH_GETATTR
@@ -1553,14 +1463,13 @@ int cs_unlink_permission(struct dentry *dentry, struct vfsmount *mnt)
 /**
  * cs_getattr_permission - Check permission for vfs_getattr().
  *
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
- * @dentry: Pointer to "struct dentry".
+ * @path: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_getattr_permission(struct vfsmount *mnt, struct dentry *dentry)
+int cs_getattr_permission(struct path *path)
 {
-	return cs_path_perm(CS_MAC_GETATTR, dentry, mnt);
+	return cs_path_perm(CS_MAC_GETATTR, path);
 }
 
 #endif
@@ -1568,70 +1477,39 @@ int cs_getattr_permission(struct vfsmount *mnt, struct dentry *dentry)
 /**
  * cs_truncate_permission - Check permission for notify_change().
  *
- * @dentry: Pointer to "struct dentry".
- * @mnt:    Pointer to "struct vfsmount". Maybe NULL.
+ * @path: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_truncate_permission(struct dentry *dentry, struct vfsmount *mnt)
+int cs_truncate_permission(struct path *path)
 {
-	return cs_path_perm(CS_MAC_TRUNCATE, dentry, mnt);
+	return cs_path_perm(CS_MAC_TRUNCATE, path);
 }
 
-#if 1
 /**
  * cs_rename_permission - Check permission for vfs_rename().
  *
- * @old_dentry: Pointer to "struct dentry".
- * @new_dentry: Pointer to "struct dentry".
- * @mnt:        Pointer to "struct vfsmount". Maybe NULL.
+ * @old: Pointer to "struct path".
+ * @new: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_rename_permission(struct dentry *old_dentry, struct dentry *new_dentry,
-			 struct vfsmount *mnt)
+int cs_rename_permission(struct path *old, struct path *new)
 {
-	return cs_path2_perm(CS_MAC_RENAME, old_dentry, mnt, new_dentry,
-			     mnt);
+	return cs_path2_perm(CS_MAC_RENAME, old, new);
 }
-#else
-/**
- * cs_rename_permission - Check permission for vfs_rename().
- *
- * @old_dentry: Pointer to "struct dentry".
- * @new_dentry: Pointer to "struct dentry".
- * @mnt:        Pointer to "struct vfsmount". Maybe NULL.
- * @flags:      Rename flags.
- *
- * Returns 0 on success, negative value otherwise.
- */
-static int __ccs_rename_permission(struct dentry *old_dentry,
-				   struct dentry *new_dentry,
-				   struct vfsmount *mnt, unsigned int flags)
-{
-	int err = cs_path2_perm(CS_MAC_RENAME, old_dentry, mnt, new_dentry,
-				mnt);
-	if (!err && (flags & RENAME_EXCHANGE))
-		err = cs_path2_perm(CS_MAC_RENAME, new_dentry, mnt, old_dentry,
-				    mnt);
-	return err;
-}
-#endif
 
 /**
  * cs_link_permission - Check permission for vfs_link().
  *
- * @old_dentry: Pointer to "struct dentry".
- * @new_dentry: Pointer to "struct dentry".
- * @mnt:        Pointer to "struct vfsmount". Maybe NULL.
+ * @old: Pointer to "struct path".
+ * @new: Pointer to "struct path".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int cs_link_permission(struct dentry *old_dentry, struct dentry *new_dentry,
-		       struct vfsmount *mnt)
+int cs_link_permission(struct path *old, struct path *new)
 {
-	return cs_path2_perm(CS_MAC_LINK, old_dentry, mnt, new_dentry,
-			     mnt);
+	return cs_path2_perm(CS_MAC_LINK, old, new);
 }
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33) && defined(CONFIG_SYSCTL_SYSCALL)
